@@ -62,7 +62,7 @@ static unsigned int nauditors;
 json_t *j_auditors;
 
 
-int
+enum GNUNET_GenericReturnValue
 TMH_AUDITORS_check_dk (struct TALER_EXCHANGE_Handle *mh,
                        const struct TALER_EXCHANGE_DenomPublicKey *dk,
                        bool exchange_trusted,
@@ -72,7 +72,7 @@ TMH_AUDITORS_check_dk (struct TALER_EXCHANGE_Handle *mh,
   const struct TALER_EXCHANGE_Keys *keys;
   const struct TALER_EXCHANGE_AuditorInformation *ai;
 
-  if (GNUNET_TIME_absolute_is_past (dk->expire_deposit))
+  if (GNUNET_TIME_absolute_is_past (dk->expire_deposit.abs_time))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Denomination key offered by client has expired for deposits\n");
@@ -120,7 +120,7 @@ TMH_AUDITORS_check_dk (struct TALER_EXCHANGE_Handle *mh,
   }
   GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
               "Denomination key %s offered by client not audited by any accepted auditor\n",
-              GNUNET_h2s (&dk->h_key));
+              GNUNET_h2s (&dk->h_key.hash));
   *hc = MHD_HTTP_BAD_REQUEST;
   *ec = TALER_EC_MERCHANT_POST_ORDERS_ID_PAY_DENOMINATION_KEY_AUDITOR_FAILURE;
   return GNUNET_NO;
