@@ -162,6 +162,7 @@ TMH_get_tips_ID (const struct TMH_RequestHandler *rh,
   struct TALER_Amount total_picked_up;
   struct GNUNET_TIME_Timestamp expiration;
   char *exchange_url;
+  char *next_url;
   struct TALER_ReservePrivateKeyP reserve_priv;
 
   (void) rh;
@@ -185,6 +186,7 @@ TMH_get_tips_ID (const struct TMH_RequestHandler *rh,
                            &total_picked_up,
                            &expiration,
                            &exchange_url,
+                           &next_url,
                            &reserve_priv);
   if (0 > qs)
   {
@@ -249,6 +251,8 @@ TMH_get_tips_ID (const struct TMH_RequestHandler *rh,
                                   &remaining),
           GNUNET_JSON_pack_string ("taler_tip_uri",
                                    uri),
+          GNUNET_JSON_pack_string ("next_url",
+                                   next_url),
           GNUNET_JSON_pack_string ("taler_tip_qrcode_svg",
                                    qr));
         ret = TALER_TEMPLATING_reply (connection,
@@ -278,12 +282,15 @@ TMH_get_tips_ID (const struct TMH_RequestHandler *rh,
         : MHD_HTTP_OK,
         GNUNET_JSON_pack_string ("exchange_url",
                                  exchange_url),
+        GNUNET_JSON_pack_string ("next_url",
+                                 next_url),
         TALER_JSON_pack_amount ("tip_amount",
                                 &remaining),
         GNUNET_JSON_pack_timestamp ("expiration",
                                     expiration));
     }
     GNUNET_free (exchange_url);
+    GNUNET_free (next_url);
     return ret;
   }
 }
