@@ -20,11 +20,12 @@
  */
 #include "platform.h"
 #include <gnunet/gnunet_util_lib.h>
+#include <gnunet/gnunet_db_lib.h>
 #include <taler/taler_util.h>
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd_helper.h"
 #include <taler/taler_templating_lib.h>
-
+#include <taler/taler_dbevents.h>
 
 /**
  * check @a payto_uris for well-formedness
@@ -560,7 +561,8 @@ trigger_webhook_cb (void *cls,
       t->rv = GNUNET_DB_STATUS_HARD_ERROR;
       return;
     }
-    GNUNET_assert ('\0' == ((const char *) header)[header_size-1]);
+    /* Note: header is actually header_size+1 bytes long here, see mustach.c::memfile_close() */
+    GNUNET_assert ('\0' == ((const char *) header)[header_size]);
   }
   if (NULL != body_template)
   {
@@ -572,13 +574,14 @@ trigger_webhook_cb (void *cls,
     if (0 != ret)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Failed to expand webhook header template for webhook %llu (%d)\n",
+                  "Failed to expand webhook body template for webhook %llu (%d)\n",
                   (unsigned long long) webhook_serial,
                   ret);
       t->rv = GNUNET_DB_STATUS_HARD_ERROR;
       return;
     }
-    GNUNET_assert ('\0' == ((const char *) body)[body_size-1]);
+    /* Note: body is actually body_size+1 bytes long here, see mustach.c::memfile_close() */
+    GNUNET_assert ('\0' == ((const char *) body)[body_size]);
   }
   t->rv = TMH_db->insert_pending_webhook (TMH_db->cls,
                                           t->instance,
@@ -587,6 +590,19 @@ trigger_webhook_cb (void *cls,
                                           http_method,
                                           header,
                                           body);
+  if (t->rv > 0)
+  {
+    struct GNUNET_DB_EventHeaderP es = {
+      .size = htons (sizeof(es)),
+      .type = htons (TALER_DBEVENT_MERCHANT_WEBHOOK_PENDING)
+    };
+    const void *extra = NULL;
+    size_t extra_size = 0;
+    TMH_db->event_notify (TMH_db->cls,
+                          &es,
+                          &extra,
+                          extra_size);
+  }
   free (header);
   free (body);
 }
