@@ -27,6 +27,7 @@
 #include <jansson.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_json_lib.h>
+#include "taler-merchant-httpd_private-get-orders.h"
 #include "taler-merchant-httpd_post-orders-ID-claim.h"
 
 
@@ -60,6 +61,8 @@ claim_order (const char *instance_id,
 {
   struct TALER_ClaimTokenP order_ct;
   enum GNUNET_DB_QueryStatus qs;
+  uint64_t order_serial;
+  bool paid = false;
 
   if (GNUNET_OK !=
       TMH_db->start (TMH_db->cls,
@@ -68,16 +71,13 @@ claim_order (const char *instance_id,
     GNUNET_break (0);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
-  {
-    uint64_t order_serial;
-
-    qs = TMH_db->lookup_contract_terms (TMH_db->cls,
-                                        instance_id,
-                                        order_id,
-                                        contract_terms,
-                                        &order_serial,
-                                        NULL);
-  }
+  qs = TMH_db->lookup_contract_terms (TMH_db->cls,
+                                      instance_id,
+                                      order_id,
+                                      contract_terms,
+                                      &order_serial,
+                                      &paid,
+                                      NULL);
   if (0 > qs)
   {
     TMH_db->rollback (TMH_db->cls);
@@ -122,9 +122,14 @@ claim_order (const char *instance_id,
   GNUNET_assert (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs);
 
   /* Now we need to claim the order. */
-
   {
-    struct GNUNET_HashCode unused;
+    struct TALER_MerchantPostDataHashP unused;
+    struct GNUNET_TIME_Timestamp timestamp;
+    struct GNUNET_JSON_Specification spec[] = {
+      GNUNET_JSON_spec_timestamp ("timestamp",
+                                  &timestamp),
+      GNUNET_JSON_spec_end ()
+    };
 
     /* see if we have this order in our table of unclaimed orders */
     qs = TMH_db->lookup_order (TMH_db->cls,
@@ -139,6 +144,19 @@ claim_order (const char *instance_id,
       return qs;
     }
     GNUNET_assert (NULL != *contract_terms);
+    if (GNUNET_OK !=
+        GNUNET_JSON_parse (*contract_terms,
+                           spec,
+                           NULL,
+                           NULL))
+    {
+      /* this should not be possible: contract_terms should always
+         have a timestamp! */
+      GNUNET_break (0);
+      TMH_db->rollback (TMH_db->cls);
+      return GNUNET_DB_STATUS_HARD_ERROR;
+    }
+
     GNUNET_assert (0 ==
                    json_object_set_new (*contract_terms,
                                         "nonce",
@@ -154,14 +172,19 @@ claim_order (const char *instance_id,
     qs = TMH_db->insert_contract_terms (TMH_db->cls,
                                         instance_id,
                                         order_id,
-                                        *contract_terms);
-    if (0 > qs)
+                                        *contract_terms,
+                                        &order_serial);
+    if (0 >= qs)
     {
       TMH_db->rollback (TMH_db->cls);
       json_decref (*contract_terms);
       *contract_terms = NULL;
       return qs;
     }
+    TMH_notify_order_change (TMH_lookup_instance (instance_id),
+                             TMH_OSF_CLAIMED,
+                             timestamp,
+                             order_serial);
     qs = TMH_db->commit (TMH_db->cls);
     if (0 > qs)
       return qs;
@@ -187,7 +210,8 @@ TMH_post_orders_ID_claim (const struct TMH_RequestHandler *rh,
                                &nonce),
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_fixed_auto ("token",
-                                     &claim_token)),
+                                     &claim_token),
+        NULL),
       GNUNET_JSON_spec_end ()
     };
     enum GNUNET_GenericReturnValue res;
@@ -196,9 +220,15 @@ TMH_post_orders_ID_claim (const struct TMH_RequestHandler *rh,
                                      hc->request_body,
                                      spec);
     if (GNUNET_OK != res)
+    {
+      GNUNET_break_op (0);
+      json_dumpf (hc->request_body,
+                  stderr,
+                  JSON_INDENT (2));
       return (GNUNET_NO == res)
              ? MHD_YES
              : MHD_NO;
+    }
   }
   contract_terms = NULL;
   for (unsigned int i = 0; i<MAX_RETRIES; i++)
@@ -243,11 +273,9 @@ TMH_post_orders_ID_claim (const struct TMH_RequestHandler *rh,
 
   /* create contract signature */
   {
+    struct TALER_PrivateContractHashP hash;
     struct GNUNET_CRYPTO_EddsaSignature merchant_sig;
-    struct TALER_ProposalDataPS pdps = {
-      .purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_CONTRACT),
-      .purpose.size = htonl (sizeof (pdps))
-    };
+
     /**
      * Hash of the JSON contract in UTF-8 including 0-termination,
      * using JSON_COMPACT | JSON_SORT_KEYS
@@ -255,7 +283,7 @@ TMH_post_orders_ID_claim (const struct TMH_RequestHandler *rh,
 
     if (GNUNET_OK !=
         TALER_JSON_contract_hash (contract_terms,
-                                  &pdps.hash))
+                                  &hash))
     {
       GNUNET_break (0);
       json_decref (contract_terms);
@@ -265,9 +293,9 @@ TMH_post_orders_ID_claim (const struct TMH_RequestHandler *rh,
                                          NULL);
     }
 
-    GNUNET_CRYPTO_eddsa_sign (&hc->instance->merchant_priv.eddsa_priv,
-                              &pdps,
-                              &merchant_sig);
+    TALER_merchant_contract_sign (&hash,
+                                  &hc->instance->merchant_priv,
+                                  &merchant_sig);
     return TALER_MHD_REPLY_JSON_PACK (
       connection,
       MHD_HTTP_OK,
