@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   (C) 2014-2022 Taler Systems SA
+   (C) 2014-2023 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
@@ -367,6 +367,16 @@ struct PayContext
    */
   struct GNUNET_TIME_Timestamp pay_deadline;
 
+  /**
+   * Set to the POS key, if applicable for this order.
+   */
+  char *pos_key;
+
+  /**
+   * Algorithm chosen for generating the confirmation code.
+   */
+  enum TALER_MerchantConfirmationAlgorithm pos_alg;
+
   /**
    * Number of transactions that the wire fees are expected to be
    * amortized over.  Never zero, defaults (conservateively) to 1.
@@ -716,6 +726,7 @@ pay_context_cleanup (void *cls)
   GNUNET_CONTAINER_DLL_remove (pc_head,
                                pc_tail,
                                pc);
+  GNUNET_free (pc->pos_key);
   GNUNET_free (pc);
 }
 
@@ -760,7 +771,8 @@ deposit_get_callback (
       NULL, /* no signature */
       NULL, /* no signature */
       now,
-      true);
+      true,
+      TALER_AML_NORMAL);
     GNUNET_break (qs > 0);
     break;
   case MHD_HTTP_ACCEPTED:
@@ -773,7 +785,8 @@ deposit_get_callback (
       NULL, /* no signature */
       NULL, /* no signature */
       now,
-      dr->details.accepted.kyc_ok);
+      dr->details.accepted.kyc_ok,
+      dr->details.accepted.aml_decision);
     GNUNET_break (qs > 0);
     break;
   default:
@@ -841,6 +854,7 @@ process_kyc_with_exchange (
  * @param exchange_url base URL of the exchange for which this is a status
  * @param last_check when did we last get an update on our KYC status from the exchange
  * @param kyc_ok true if we satisfied the KYC requirements
+ * @param aml_decision latest AML decision by the exchange
  */
 static void
 kyc_cb (
@@ -850,12 +864,18 @@ kyc_cb (
   const char *payto_uri,
   const char *exchange_url,
   struct GNUNET_TIME_Timestamp last_check,
-  bool kyc_ok)
+  bool kyc_ok,
+  enum TALER_AmlDecisionState aml_decision)
 {
   struct KycContext *kc = cls;
 
+  (void) h_wire;
+  (void) exchange_kyc_serial;
+  (void) payto_uri;
+  (void) exchange_url;
   kc->kyc_timestamp = last_check;
   kc->kyc_ok = kyc_ok;
+  /* FIXME: act on aml_decision? */
 }
 
 
@@ -1938,6 +1958,7 @@ static void
 generate_success_response (struct PayContext *pc)
 {
   struct GNUNET_CRYPTO_EddsaSignature sig;
+  char *pos_confirmation;
 
   /* Sign on our end (as the payment did go through, even if it may
      have been refunded already) */
@@ -1945,12 +1966,22 @@ generate_success_response (struct PayContext *pc)
                            &pc->hc->instance->merchant_priv,
                            &sig);
   /* Build the response */
+  pos_confirmation = (NULL == pc->pos_key)
+    ? NULL
+    : TALER_build_pos_confirmation (pc->pos_key,
+                                    pc->pos_alg,
+                                    &pc->total_paid,
+                                    pc->timestamp);
   resume_pay_with_response (
     pc,
     MHD_HTTP_OK,
     TALER_MHD_MAKE_JSON_PACK (
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("pos_confirmation",
+                                 pos_confirmation)),
       GNUNET_JSON_pack_data_auto ("sig",
                                   &sig)));
+  GNUNET_free (pos_confirmation);
 }
 
 
@@ -2301,6 +2332,7 @@ parse_pay (struct PayContext *pc)
                            &pc->dc[j].cdd.coin_pub))
         {
           GNUNET_break_op (0);
+          GNUNET_JSON_parse_free (spec);
           return (MHD_YES ==
                   TALER_MHD_reply_with_error (pc->connection,
                                               MHD_HTTP_BAD_REQUEST,
@@ -2353,7 +2385,7 @@ parse_pay (struct PayContext *pc)
       {
         if (0 ==
             strcmp (pc->egs[i]->exchange_url,
-                    GNUNET_strdup (exchange_url)))
+                    exchange_url))
         {
           have_eg = true;
           break;
@@ -2547,13 +2579,15 @@ check_contract (struct PayContext *pc)
     json_decref (pc->contract_terms);
     pc->contract_terms = NULL;
   }
-  qs = TMH_db->lookup_contract_terms (TMH_db->cls,
-                                      pc->hc->instance->settings.id,
-                                      pc->order_id,
-                                      &pc->contract_terms,
-                                      &pc->order_serial,
-                                      &paid,
-                                      NULL);
+  qs = TMH_db->lookup_contract_terms2 (TMH_db->cls,
+                                       pc->hc->instance->settings.id,
+                                       pc->order_id,
+                                       &pc->contract_terms,
+                                       &pc->order_serial,
+                                       &paid,
+                                       NULL,
+                                       &pc->pos_key,
+                                       &pc->pos_alg);
   if (0 > qs)
   {
     /* single, read-only SQL statements should never cause
