@@ -619,6 +619,7 @@ TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
 {
   struct PickupContext *pc = hc->ctx;
   char *exchange_url;
+  char *next_url;
   struct TALER_Amount total_authorized;
   struct TALER_Amount total_picked_up;
   struct TALER_Amount total_remaining;
@@ -740,6 +741,8 @@ TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
 
   if (! pc->tr_initialized)
   {
+    char *next_url;
+
     qs = TMH_db->lookup_tip (TMH_db->cls,
                              hc->instance->settings.id,
                              &pc->tip_id,
@@ -747,10 +750,12 @@ TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
                              &total_picked_up,
                              &expiration,
                              &exchange_url,
+                             &next_url,
                              &pc->reserve_priv);
     if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
       return reply_lookup_tip_failed (connection,
                                       qs);
+    GNUNET_free (next_url);
     MHD_suspend_connection (connection);
     pc->connection = connection;
     pc->tt = GNUNET_SCHEDULER_add_delayed (EXCHANGE_TIMEOUT,
@@ -900,6 +905,7 @@ RETRY:
                            &total_picked_up,
                            &expiration,
                            &exchange_url,
+                           &next_url,
                            &pc->reserve_priv);
   if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
   {
@@ -912,6 +918,7 @@ RETRY:
     return reply_lookup_tip_failed (connection,
                                     qs);
   }
+  GNUNET_free (next_url);
   if (GNUNET_TIME_absolute_is_past (expiration.abs_time))
   {
     GNUNET_free (exchange_url);
