@@ -45,7 +45,7 @@ delete_instances_ID (struct TMH_MerchantInstance *mi,
   if (NULL == purge_s)
     purge_s = "no";
   purge = (0 == strcasecmp (purge_s,
-                        "yes"));
+                            "yes"));
   if (purge)
     qs = TMH_db->purge_instance (TMH_db->cls,
                                  mi->settings.id);
@@ -74,12 +74,6 @@ delete_instances_ID (struct TMH_MerchantInstance *mi,
                                        : "Private key unknown");
   case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
     TMH_reload_instances (mi->settings.id);
-    if (purge)
-      TMH_instance_free_cb (NULL,
-                            &mi->h_instance,
-                            mi);
-    else
-      mi->deleted = true;
     return TALER_MHD_reply_static (connection,
                                    MHD_HTTP_NO_CONTENT,
                                    NULL,
@@ -98,6 +92,7 @@ TMH_private_delete_instances_ID (const struct TMH_RequestHandler *rh,
 {
   struct TMH_MerchantInstance *mi = hc->instance;
 
+  (void) rh;
   return delete_instances_ID (mi,
                               connection);
 }
@@ -110,6 +105,7 @@ TMH_private_delete_instances_default_ID (const struct TMH_RequestHandler *rh,
 {
   struct TMH_MerchantInstance *mi;
 
+  (void) rh;
   mi = TMH_lookup_instance (hc->infix);
   if (NULL == mi)
   {
