@@ -110,11 +110,6 @@ struct ExchangeKycRequest
    */
   struct GNUNET_TIME_Timestamp last_check;
 
-  /**
-   * Last KYC status returned by the exchange.
-   */
-  bool kyc_ok;
-
 };
 
 
@@ -310,9 +305,9 @@ kyc_context_cleanup (void *cls)
 
 
 /**
- * Resume the given KYC context and send the given response.
- * Stores the response in the @a kc and signals MHD to resume
- * the connection.  Also ensures MHD runs immediately.
+ * Resume the given KYC context and send the given response.  Stores the
+ * response in the @a kc and signals MHD to resume the connection.  Also
+ * ensures MHD runs immediately.
  *
  * @param kc KYC context
  * @param response_code response code to use
@@ -433,9 +428,8 @@ handle_kyc_timeout (void *cls)
 
 
 /**
- * We are done with the KYC request @a ekr.
- * Remove it from the work list and check if
- * we are done overall.
+ * We are done with the KYC request @a ekr.  Remove it from the work list and
+ * check if we are done overall.
  *
  * @param[in] ekr key request that is done (and will be freed)
  */
@@ -494,6 +488,7 @@ exchange_check_cb (void *cls,
   struct KycContext *kc = ekr->kc;
 
   ekr->kyc = NULL;
+  // FIXME: handle case where exchange returns that account is AML pending/frozen!
   switch (ks->http_status)
   {
   case MHD_HTTP_OK:
@@ -508,7 +503,8 @@ exchange_check_cb (void *cls,
                                            &ks->details.success.exchange_sig,
                                            &ks->details.success.exchange_pub,
                                            ks->details.success.timestamp,
-                                           true);
+                                           true,
+                                           TALER_AML_NORMAL);
       if (qs < 0)
       {
         GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -543,7 +539,8 @@ exchange_check_cb (void *cls,
                                            NULL,
                                            NULL,
                                            now,
-                                           true);
+                                           true,
+                                           TALER_AML_NORMAL);
       if (qs < 0)
       {
         GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
@@ -636,8 +633,8 @@ kyc_with_exchange (void *cls,
 
 
 /**
- * Function called from ``account_kyc_get_status``
- * with KYC status information for this merchant.
+ * Function called from account_kyc_get_status() with KYC status information
+ * for this merchant.
  *
  * @param cls our `struct KycContext *`
  * @param h_wire hash of the wire account
@@ -646,6 +643,7 @@ kyc_with_exchange (void *cls,
  * @param exchange_url base URL of the exchange for which this is a status
  * @param last_check when did we last get an update on our KYC status from the exchange
  * @param kyc_ok true if we satisfied the KYC requirements
+ * @param aml_decision latest AML decision known to us
  */
 static void
 kyc_status_cb (void *cls,
@@ -654,12 +652,14 @@ kyc_status_cb (void *cls,
                const char *payto_uri,
                const char *exchange_url,
                struct GNUNET_TIME_Timestamp last_check,
-               bool kyc_ok)
+               bool kyc_ok,
+               enum TALER_AmlDecisionState aml_decision)
 {
   struct KycContext *kc = cls;
   struct ExchangeKycRequest *ekr;
 
   if (kyc_ok &&
+      (TALER_AML_PENDING != aml_decision) &&
       (GNUNET_TIME_relative_cmp (
          GNUNET_TIME_absolute_get_duration (last_check.abs_time),
          <,
@@ -680,7 +680,6 @@ kyc_status_cb (void *cls,
   ekr->exchange_url = GNUNET_strdup (exchange_url);
   ekr->payto_uri = GNUNET_strdup (payto_uri);
   ekr->last_check = last_check;
-  ekr->kyc_ok = kyc_ok;
   ekr->kc = kc;
   ekr->fo = TMH_EXCHANGES_find_exchange (exchange_url,
                                          NULL,
