@@ -537,7 +537,8 @@ deposit_get_cb (void *cls,
         NULL,
         NULL,
         now,
-        false);
+        dr->details.accepted.kyc_ok,
+        dr->details.accepted.aml_decision);
       if (qs < 0)
       {
         gorc_report (gorc,
