@@ -33,7 +33,7 @@
 static void
 add_tip (void *cls,
          uint64_t row_id,
-         struct GNUNET_HashCode tip_id,
+         struct TALER_TipIdentifierP tip_id,
          struct TALER_Amount amount)
 {
   json_t *pa = cls;
@@ -62,6 +62,7 @@ TMH_private_get_tips (const struct TMH_RequestHandler *rh,
   uint64_t offset;
   int64_t limit;
 
+  (void) rh;
   if (! (TALER_arg_to_yna (connection,
                            "expired",
                            TALER_EXCHANGE_YNA_NO,
