@@ -52,9 +52,9 @@ determine_cause (struct MHD_Connection *connection,
   enum GNUNET_DB_QueryStatus qs;
 
   qs = TMH_db->lookup_template (TMH_db->cls,
-                               instance_id,
-                               template_id,
-                               &tpx);
+                                instance_id,
+                                template_id,
+                                &tpx);
   switch (qs)
   {
   case GNUNET_DB_STATUS_HARD_ERROR:
@@ -102,19 +102,24 @@ determine_cause (struct MHD_Connection *connection,
  */
 MHD_RESULT
 TMH_private_patch_templates_ID (const struct TMH_RequestHandler *rh,
-                               struct MHD_Connection *connection,
-                               struct TMH_HandlerContext *hc)
+                                struct MHD_Connection *connection,
+                                struct TMH_HandlerContext *hc)
 {
   struct TMH_MerchantInstance *mi = hc->instance;
   const char *template_id = hc->infix;
   struct TALER_MERCHANTDB_TemplateDetails tp = {0};
   enum GNUNET_DB_QueryStatus qs;
+  uint32_t pos_algorithm;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("template_description",
                              (const char **) &tp.template_description),
     GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_string ("image",
-                               (const char **) &tp.image),
+      GNUNET_JSON_spec_uint32 ("pos_algorithm",
+                               &pos_algorithm),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("pos_key",
+                               (const char **) &tp.pos_key),
       NULL),
     GNUNET_JSON_spec_json ("template_contract",
                            &tp.template_contract),
@@ -135,7 +140,8 @@ TMH_private_patch_templates_ID (const struct TMH_RequestHandler *rh,
              : MHD_NO;
   }
 
-    if (! TMH_template_contract_valid (tp.template_contract))
+  tp.pos_algorithm = (enum TALER_MerchantConfirmationAlgorithm) pos_algorithm;
+  if (! TMH_template_contract_valid (tp.template_contract))
   {
     GNUNET_break_op (0);
     GNUNET_JSON_parse_free (spec);
@@ -145,21 +151,10 @@ TMH_private_patch_templates_ID (const struct TMH_RequestHandler *rh,
                                        "template_contract");
   }
 
-
-  if (! TMH_image_data_url_valid (tp.image))
-  {
-    GNUNET_break_op (0);
-    GNUNET_JSON_parse_free (spec);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                       "image");
-  }
-
   qs = TMH_db->update_template (TMH_db->cls,
-                               mi->settings.id,
-                               template_id,
-                               &tp);
+                                mi->settings.id,
+                                template_id,
+                                &tp);
   {
     MHD_RESULT ret = MHD_NO;
 
