@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014, 2015, 2016, 2018, 2020, 2021 Taler Systems SA
+  (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -244,6 +244,8 @@ struct InventoryProduct
  * @param uuids_length length of the @a uuids array
  * @param uuids array of UUIDs used to reserve products from @a inventory_products
  * @param[out] out_of_stock_index which product (by offset) is out of stock, UINT_MAX if all were in-stock
+ * @param pos_key shared secret with the POS
+ * @param pos_algorithm algorithm for payment confirmation generation
  * @return transaction status, 0 if @a uuids were insufficient to reserve required inventory
  */
 static enum GNUNET_DB_QueryStatus
@@ -257,7 +259,9 @@ execute_transaction (struct TMH_HandlerContext *hc,
                      const struct InventoryProduct inventory_products[],
                      unsigned int uuids_length,
                      const struct GNUNET_Uuid uuids[],
-                     unsigned int *out_of_stock_index)
+                     unsigned int *out_of_stock_index,
+                     const char *pos_key,
+                     enum TALER_MerchantConfirmationAlgorithm pos_algorithm)
 {
   enum GNUNET_DB_QueryStatus qs;
   struct GNUNET_TIME_Timestamp timestamp;
@@ -277,7 +281,9 @@ execute_transaction (struct TMH_HandlerContext *hc,
                              h_post_data,
                              pay_deadline,
                              claim_token,
-                             order); // called 'contract terms' at database.
+                             order,  /* called 'contract terms' at database. */
+                             pos_key,
+                             pos_algorithm);
   if (qs <= 0)
   {
     /* qs == 0: probably instance does not exist (anymore) */
@@ -362,6 +368,8 @@ execute_transaction (struct TMH_HandlerContext *hc,
  * @param inventory_products array of products to add to @a order from our inventory
  * @param uuids_length length of the @a uuids array
  * @param uuids array of UUIDs used to reserve products from @a inventory_products
+ * @param pos_key encoded key for verification payment
+ * @param pos_algorithm  algorithm to compute the payment verification
  * @return MHD result code
  */
 static MHD_RESULT
@@ -373,7 +381,9 @@ execute_order (struct MHD_Connection *connection,
                unsigned int inventory_products_length,
                const struct InventoryProduct inventory_products[],
                unsigned int uuids_length,
-               const struct GNUNET_Uuid uuids[])
+               const struct GNUNET_Uuid uuids[],
+               const char *pos_key,
+               enum TALER_MerchantConfirmationAlgorithm pos_algorithm)
 {
   const struct TALER_MERCHANTDB_InstanceSettings *settings =
     &hc->instance->settings;
@@ -553,7 +563,9 @@ execute_order (struct MHD_Connection *connection,
                               inventory_products,
                               uuids_length,
                               uuids,
-                              &out_of_stock_index);
+                              &out_of_stock_index,
+                              pos_key,
+                              pos_algorithm);
     if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
       break;
   }
@@ -691,6 +703,8 @@ execute_order (struct MHD_Connection *connection,
  * @param inventory_products array of products to add to @a order from our inventory
  * @param uuids_length length of the @a uuids array
  * @param uuids array of UUIDs used to reserve products from @a inventory_products
+ * @param pos_key encoded key for verification payment
+ * @param pos_algorithm  algorithm to compute the payment verification
  * @return MHD result code
  */
 static MHD_RESULT
@@ -703,7 +717,9 @@ patch_order (struct MHD_Connection *connection,
              unsigned int inventory_products_length,
              const struct InventoryProduct inventory_products[],
              unsigned int uuids_length,
-             const struct GNUNET_Uuid uuids[])
+             const struct GNUNET_Uuid uuids[],
+             const char *pos_key,
+             enum TALER_MerchantConfirmationAlgorithm pos_algorithm)
 {
   const struct TALER_MERCHANTDB_InstanceSettings *settings =
     &hc->instance->settings;
@@ -916,13 +932,12 @@ patch_order (struct MHD_Connection *connection,
       {
         GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                     "Refund delay is zero, no refunds are possible for this order\n");
-        refund_deadline = now; /* if delay was 0, ensure that refund_deadline == timestamp */
+        refund_deadline = GNUNET_TIME_UNIT_ZERO_TS;
       }
       else
       {
         refund_deadline = GNUNET_TIME_relative_to_timestamp (refund_delay);
       }
-
       GNUNET_assert (0 ==
                      json_object_set_new (order,
                                           "refund_deadline",
@@ -930,9 +945,7 @@ patch_order (struct MHD_Connection *connection,
                                             refund_deadline)));
     }
     if ( (! GNUNET_TIME_absolute_is_zero (delivery_date.abs_time)) &&
-         (GNUNET_TIME_timestamp_cmp (delivery_date,
-                                     <,
-                                     now)) )
+         (GNUNET_TIME_absolute_is_past (delivery_date.abs_time)) )
     {
       GNUNET_break_op (0);
       GNUNET_JSON_parse_free (spec);
@@ -954,6 +967,27 @@ patch_order (struct MHD_Connection *connection,
                                         "pay_deadline",
                                         GNUNET_JSON_from_timestamp (t)));
   }
+  else if (GNUNET_TIME_absolute_is_past (pay_deadline.abs_time))
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    return TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_BAD_REQUEST,
+      TALER_EC_MERCHANT_PRIVATE_POST_ORDERS_PAY_DEADLINE_IN_PAST,
+      NULL);
+  }
+  if ( (! GNUNET_TIME_absolute_is_zero (refund_deadline.abs_time)) &&
+       (GNUNET_TIME_absolute_is_past (refund_deadline.abs_time)) )
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    return TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_BAD_REQUEST,
+      TALER_EC_MERCHANT_PRIVATE_POST_ORDERS_REFUND_DEADLINE_IN_PAST,
+      NULL);
+  }
 
   if (GNUNET_TIME_absolute_is_never (wire_deadline.abs_time))
   {
@@ -1185,7 +1219,9 @@ patch_order (struct MHD_Connection *connection,
                           inventory_products_length,
                           inventory_products,
                           uuids_length,
-                          uuids);
+                          uuids,
+                          pos_key,
+                          pos_algorithm);
     GNUNET_JSON_parse_free (spec);
     return mres;
   }
@@ -1208,6 +1244,8 @@ patch_order (struct MHD_Connection *connection,
  * @param inventory_products array of products to add to @a order from our inventory
  * @param uuids_length length of the @a uuids array
  * @param uuids array of UUIDs used to reserve products from @a inventory_products
+ * @param pos_key encoded key for verification payment
+ * @param pos_algorithm  algorithm to compute the payment verification
  * @return MHD result code
  */
 static MHD_RESULT
@@ -1221,7 +1259,9 @@ add_payment_details (struct MHD_Connection *connection,
                      unsigned int inventory_products_length,
                      const struct InventoryProduct inventory_products[],
                      unsigned int uuids_length,
-                     const struct GNUNET_Uuid uuids[])
+                     const struct GNUNET_Uuid uuids[],
+                     const char *pos_key,
+                     enum TALER_MerchantConfirmationAlgorithm pos_algorithm)
 {
   struct TMH_WireMethod *wm;
 
@@ -1261,7 +1301,9 @@ add_payment_details (struct MHD_Connection *connection,
                       inventory_products_length,
                       inventory_products,
                       uuids_length,
-                      uuids);
+                      uuids,
+                      pos_key,
+                      pos_algorithm);
 }
 
 
@@ -1281,6 +1323,8 @@ add_payment_details (struct MHD_Connection *connection,
  * @param inventory_products array of products to add to @a order from our inventory
  * @param uuids_length length of the @a uuids array
  * @param uuids array of UUIDs used to reserve products from @a inventory_products
+ * @param pos_key encoded key for verification payment
+ * @param pos_algorithm  algorithm to compute the payment verification
  * @return MHD result code
  */
 static MHD_RESULT
@@ -1294,7 +1338,9 @@ merge_inventory (struct MHD_Connection *connection,
                  unsigned int inventory_products_length,
                  const struct InventoryProduct inventory_products[],
                  unsigned int uuids_length,
-                 const struct GNUNET_Uuid uuids[])
+                 const struct GNUNET_Uuid uuids[],
+                 const char *pos_key,
+                 enum TALER_MerchantConfirmationAlgorithm pos_algorithm)
 {
   /**
    * inventory_products => instructions to add products to contract terms
@@ -1417,24 +1463,19 @@ merge_inventory (struct MHD_Connection *connection,
                               inventory_products_length,
                               inventory_products,
                               uuids_length,
-                              uuids);
+                              uuids,
+                              pos_key,
+                              pos_algorithm);
 }
 
 
-/**
- * Generate an order.  We add the fields 'exchanges', 'merchant_pub', and
- * 'H_wire' to the order gotten from the frontend, as well as possibly other
- * fields if the frontend did not provide them. Returns the order_id.
- *
- * @param rh context of the handler
- * @param connection the MHD connection to handle
- * @param[in,out] hc context with further information about the request
- * @return MHD result code
- */
 MHD_RESULT
-TMH_private_post_orders (const struct TMH_RequestHandler *rh,
-                         struct MHD_Connection *connection,
-                         struct TMH_HandlerContext *hc)
+TMH_private_post_orders_with_pos_secrets (
+  const struct TMH_RequestHandler *rh,
+  struct MHD_Connection *connection,
+  struct TMH_HandlerContext *hc,
+  const char *pos_key,
+  enum TALER_MerchantConfirmationAlgorithm pos_algorithm)
 {
   json_t *order;
   struct GNUNET_TIME_Relative refund_delay = GNUNET_TIME_UNIT_ZERO;
@@ -1637,7 +1678,9 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
                            ips_len,
                            ips,
                            uuids_len,
-                           uuids);
+                           uuids,
+                           pos_key,
+                           pos_algorithm);
     GNUNET_array_grow (ips,
                        ips_len,
                        0);
@@ -1650,4 +1693,17 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
 }
 
 
+MHD_RESULT
+TMH_private_post_orders (const struct TMH_RequestHandler *rh,
+                         struct MHD_Connection *connection,
+                         struct TMH_HandlerContext *hc)
+{
+  return TMH_private_post_orders_with_pos_secrets (rh,
+                                                   connection,
+                                                   hc,
+                                                   NULL,
+                                                   TALER_MCA_NONE);
+}
+
+
 /* end of taler-merchant-httpd_private-post-orders.c */
