@@ -44,8 +44,8 @@ authorize_tip (const struct TMH_RequestHandler *rh,
                const struct TALER_ReservePublicKeyP *reserve_pub)
 {
   enum TALER_ErrorCode ec;
-  struct GNUNET_TIME_Absolute expiration;
-  struct GNUNET_HashCode tip_id;
+  struct GNUNET_TIME_Timestamp expiration;
+  struct TALER_TipIdentifierP tip_id;
   const char *justification;
   const char *next_url;
   struct TALER_Amount amount;
@@ -119,7 +119,7 @@ authorize_tip (const struct TMH_RequestHandler *rh,
     struct GNUNET_CRYPTO_HashAsciiEncoded hash_enc;
     MHD_RESULT res;
 
-    GNUNET_CRYPTO_hash_to_enc (&tip_id,
+    GNUNET_CRYPTO_hash_to_enc (&tip_id.hash,
                                &hash_enc);
     taler_tip_uri = TMH_make_taler_tip_uri (connection,
                                             &tip_id,
@@ -127,7 +127,6 @@ authorize_tip (const struct TMH_RequestHandler *rh,
     tip_status_url = TMH_make_tip_status_url (connection,
                                               &tip_id,
                                               hc->instance->settings.id);
-    GNUNET_TIME_round_abs (&expiration);
     res = TALER_MHD_REPLY_JSON_PACK (
       connection,
       MHD_HTTP_OK,
@@ -137,8 +136,8 @@ authorize_tip (const struct TMH_RequestHandler *rh,
                                taler_tip_uri),
       GNUNET_JSON_pack_string ("tip_status_url",
                                tip_status_url),
-      GNUNET_JSON_pack_time_abs ("tip_expiration",
-                                 expiration));
+      GNUNET_JSON_pack_timestamp ("tip_expiration",
+                                  expiration));
     GNUNET_free (taler_tip_uri);
     GNUNET_free (tip_status_url);
     return res;
@@ -162,7 +161,7 @@ TMH_private_post_reserves_ID_authorize_tip (const struct TMH_RequestHandler *rh,
     GNUNET_break_op (0);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_MERCHANT_GENERIC_RESERVE_PUB_MALFORMED,
+                                       TALER_EC_GENERIC_RESERVE_PUB_MALFORMED,
                                        hc->infix);
   }
   return authorize_tip (rh,
