@@ -47,10 +47,12 @@ templates_equal (const struct TALER_MERCHANTDB_TemplateDetails *t1,
 {
   return ( (0 == strcmp (t1->template_description,
                          t2->template_description)) &&
-           ( ( (NULL == t1->image) && (NULL == t2->image) ) ||
-             ( (NULL != t1->image) && (NULL != t2->image) &&
-               (0 == strcmp (t1->image,
-                             t2->image)) ) ) &&
+           ( ( (NULL == t1->pos_key) &&
+               (NULL == t2->pos_key) ) ||
+             ( (NULL != t1->pos_key) &&
+               (NULL != t2->pos_key) &&
+               (0 == strcmp (t1->pos_key,
+                             t2->pos_key))) ) &&
            (1 == json_equal (t1->template_contract,
                              t2->template_contract)) );
 }
@@ -65,14 +67,19 @@ TMH_private_post_templates (const struct TMH_RequestHandler *rh,
   struct TALER_MERCHANTDB_TemplateDetails tp = { 0 };
   const char *template_id;
   enum GNUNET_DB_QueryStatus qs;
+  uint32_t pos_algorithm = 0;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("template_id",
                              &template_id),
     GNUNET_JSON_spec_string ("template_description",
                              (const char **) &tp.template_description),
     GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_string ("image",
-                               (const char **) &tp.image),
+      GNUNET_JSON_spec_uint32 ("pos_algorithm",
+                               &pos_algorithm),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("pos_key",
+                               (const char **) &tp.pos_key),
       NULL),
     GNUNET_JSON_spec_json ("template_contract",
                            &tp.template_contract),
@@ -94,6 +101,7 @@ TMH_private_post_templates (const struct TMH_RequestHandler *rh,
              : MHD_NO;
     }
   }
+  tp.pos_algorithm = (enum TALER_MerchantConfirmationAlgorithm) pos_algorithm;
   if (! TMH_template_contract_valid (tp.template_contract))
   {
     GNUNET_break_op (0);
@@ -107,17 +115,6 @@ TMH_private_post_templates (const struct TMH_RequestHandler *rh,
                                        "template_contract");
   }
 
-  if ( (NULL != tp.image) &&
-       (! TMH_image_data_url_valid (tp.image)) )
-  {
-    GNUNET_break_op (0);
-    GNUNET_JSON_parse_free (spec);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                       "image");
-  }
-
   /* finally, interact with DB until no serialization error */
   for (unsigned int i = 0; i<MAX_RETRIES; i++)
   {
@@ -194,8 +191,8 @@ TMH_private_post_templates (const struct TMH_RequestHandler *rh,
       if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
         break;
     }
-retry: 
-   GNUNET_assert (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+retry:
+    GNUNET_assert (GNUNET_DB_STATUS_SOFT_ERROR == qs);
     TMH_db->rollback (TMH_db->cls);
   } /* for RETRIES loop */
   GNUNET_JSON_parse_free (spec);
