@@ -43,7 +43,7 @@ TALER_MERCHANTDB_template_details_free (
   struct TALER_MERCHANTDB_TemplateDetails *tp)
 {
   GNUNET_free (tp->template_description);
-  GNUNET_free (tp->image);
+  GNUNET_free (tp->pos_key);
   json_decref (tp->template_contract);
 }
 
