@@ -1767,6 +1767,62 @@ TALER_TESTING_cmd_merchant_delete_webhook (const char *label,
                                            const char *webhook_id,
                                            unsigned int http_status);
 
+/**
+ * to use the 'taler-merchant-webhook' program.
+ *
+ * @param label command label.
+ * @param config_filename configuration file used by the webhook.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_webhook (const char *label,
+                           const char *config_filename);
+
+
+/**
+ * This function is used to start the web server.
+ *
+ * @param label command label
+ * @param port is the port of the web server
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_testserver (const char *label,
+                              uint16_t port);
+
+
+/**
+ * This function is used to check the web server got the
+ * expected request from the web hook.
+ *
+ * @param label command label
+ * @param ref_operation reference to command to the previous set server status operation.
+ * @param index index to know which web server we check.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_checkserver (const char *label,
+                               const char *ref_operation,
+                               unsigned int index);
+
+
+/**
+ * This function is used to check that the web server
+ * got the expected request from the web hook.
+ *
+ * @param label command label
+ * @param ref_operation reference to command to the previous set server status operation.
+ * @param index index to know which web server we check.
+ * @param expected_url url of the webhook
+ * @param expected_method method of the webhook
+ * @param expected_header header of the webhook
+ * @param expected_body body of the webhook
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_checkserver2 (const char *label,
+                                const char *ref_operation,
+                                unsigned int index,
+                                const char *expected_url,
+                                const char *expected_method,
+                                const char *expected_header,
+                                const char *expected_body);
 
 /* ****** Specific traits supported by this component ******* */
 
@@ -1810,12 +1866,12 @@ TALER_TESTING_cmd_merchant_delete_webhook (const char *label,
   op (h_wire, const struct TALER_MerchantWireHashP) \
   op (proposal_reference, const char *) \
   op (template_description, const char *) \
-  op (template_image, const char *) \
+  op (template_pos_key, const char *) \
   op (template_id, const char *) \
   op (template_contract, const json_t) \
   op (event_type, const char *)   \
-  op (url, const char *) \
   op (webhook_id, const char *) \
+  op (url, const char *) \
   op (http_method, const char *)   \
   op (header_template, const char *)   \
   op (body_template, const char *)   \
@@ -1832,6 +1888,11 @@ TALER_TESTING_cmd_merchant_delete_webhook (const char *label,
   op (paths, const char *) \
   op (payto_uris, const char *) \
   op (amounts, const struct TALER_Amount) \
+  op (urls, char *) \
+  op (http_methods, char *)   \
+  op (http_header, char *)   \
+  op (http_body, void *)   \
+  op (http_body_size, size_t)   \
   op (planchet_secrets, const struct TALER_PlanchetMasterSecretP)
 
 
