@@ -151,6 +151,7 @@ struct TALER_MERCHANTDB_InstanceSettings
    * merchant's logo data uri
    */
   char *logo;
+
   /**
    * Address of the business
    */
@@ -326,14 +327,19 @@ struct TALER_MERCHANTDB_TemplateDetails
   char *template_description;
 
   /**
-   * Base64-encoded image, or NULL.
+   * Base64-encoded key, or NULL.
    */
-  char *image;
+  char *pos_key;
 
   /**
    * In this template contract, we can have additional information.
    */
   json_t *template_contract;
+
+  /**
+   * Algorithm used to compute purchase confirmations.
+   */
+  enum TALER_MerchantConfirmationAlgorithm pos_algorithm;
 };
 
 
@@ -409,7 +415,7 @@ typedef void
  * Typically called by `lookup_pending_webhooks`.
  *
  * @param cls a `json_t *` JSON array to build
- * @param webhook_serial reference to the configured webhook template.
+ * @param webhook_pending_serial reference to the configured webhook template.
  * @param next_attempt is the time we should make the next request to the webhook.
  * @param retries how often have we tried this request to the webhook.
  * @param url to make request to
@@ -419,7 +425,7 @@ typedef void
  */
 typedef void
 (*TALER_MERCHANTDB_PendingWebhooksCallback)(void *cls,
-                                            uint64_t webhook_serial,
+                                            uint64_t webhook_pending_serial,
                                             struct GNUNET_TIME_Absolute
                                             next_attempt,
                                             uint32_t retries,
@@ -434,6 +440,7 @@ typedef void
  */
 struct TALER_MERCHANTDB_PendingWebhookDetails
 {
+
   /**
    * Identifies when we should make the next request to the webhook. 0 for unknown,
    * #GNUNET_TIME_UNIT_FOREVER_ABS for never.
@@ -455,13 +462,11 @@ struct TALER_MERCHANTDB_PendingWebhookDetails
    */
   char *http_method;
 
-
   /**
    * Header of the webhook.
    */
   char *header;
 
-
   /**
    * Body of the webhook.
    */
@@ -625,6 +630,7 @@ typedef void
  * @param exchange_url base URL of the exchange for which this is a status
  * @param last_check when did we last get an update on our KYC status from the exchange
  * @param kyc_ok true if we satisfied the KYC requirements
+ * @param aml_decision current AML decision state at the exchange
  */
 typedef void
 (*TALER_MERCHANTDB_KycCallback)(
@@ -634,7 +640,8 @@ typedef void
   const char *payto_uri,
   const char *exchange_url,
   struct GNUNET_TIME_Timestamp last_check,
-  bool kyc_ok);
+  bool kyc_ok,
+  enum TALER_AmlDecisionState aml_decision);
 
 
 /**
@@ -1250,6 +1257,7 @@ struct TALER_MERCHANTDB_Plugin
    * @param exchange_pub public key of the exchange, or NULL for none
    * @param timestamp timestamp to store
    * @param kyc_ok current KYC status (true for satisfied)
+   * @param aml_decision current AML decision state at the exchange
    * @return database result code
    */
   enum GNUNET_DB_QueryStatus
@@ -1261,7 +1269,8 @@ struct TALER_MERCHANTDB_Plugin
                             const struct TALER_ExchangeSignatureP *exchange_sig,
                             const struct TALER_ExchangePublicKeyP *exchange_pub,
                             struct GNUNET_TIME_Timestamp timestamp,
-                            bool kyc_ok);
+                            bool kyc_ok,
+                            enum TALER_AmlDecisionState aml_decision);
 
 
   /**
@@ -1469,6 +1478,8 @@ struct TALER_MERCHANTDB_Plugin
    * @param pay_deadline how long does the customer have to pay for the order
    * @param claim_token token to use for access control
    * @param contract_terms proposal data to store
+   * @param pos_key encoded key for payment verification
+   * @param pos_algorithm algorithm to compute the payment verification
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
@@ -1478,7 +1489,9 @@ struct TALER_MERCHANTDB_Plugin
                   const struct TALER_MerchantPostDataHashP *h_post_data,
                   struct GNUNET_TIME_Timestamp pay_deadline,
                   const struct TALER_ClaimTokenP *claim_token,
-                  const json_t *contract_terms);
+                  const json_t *contract_terms,
+                  const char *pos_key,
+                  enum TALER_MerchantConfirmationAlgorithm pos_algorithm);
 
 
   /**
@@ -1526,16 +1539,44 @@ struct TALER_MERCHANTDB_Plugin
    * @param[out] order_serial set to the order's serial number
    * @param[out] paid set to true if the order is fully paid
    * @param[out] claim_token set to the claim token, NULL to only check for existence
+   * @param[out] pos_key encoded key for payment verification
+   * @param[out] pos_algorithm set to algorithm to compute the payment verification
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*lookup_contract_terms)(void *cls,
-                           const char *instance_id,
-                           const char *order_id,
-                           json_t **contract_terms,
-                           uint64_t *order_serial,
-                           bool *paid,
-                           struct TALER_ClaimTokenP *claim_token);
+  (*lookup_contract_terms2)(
+    void *cls,
+    const char *instance_id,
+    const char *order_id,
+    json_t **contract_terms,
+    uint64_t *order_serial,
+    bool *paid,
+    struct TALER_ClaimTokenP *claim_token,
+    char **pos_key,
+    enum TALER_MerchantConfirmationAlgorithm *pricing_algorithm);
+
+
+  /**
+   * Retrieve contract terms given its @a order_id
+   *
+   * @param cls closure
+   * @param instance_id instance's identifier
+   * @param order_id order_id used to lookup.
+   * @param[out] contract_terms where to store the result, NULL to only check for existence
+   * @param[out] order_serial set to the order's serial number
+   * @param[out] paid set to true if the order is fully paid
+   * @param[out] claim_token set to the claim token, NULL to only check for existence
+   * @return transaction status
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_contract_terms)(
+    void *cls,
+    const char *instance_id,
+    const char *order_id,
+    json_t **contract_terms,
+    uint64_t *order_serial,
+    bool *paid,
+    struct TALER_ClaimTokenP *claim_token);
 
 
   /**
@@ -1555,11 +1596,12 @@ struct TALER_MERCHANTDB_Plugin
    *          is malformed
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_contract_terms)(void *cls,
-                           const char *instance_id,
-                           const char *order_id,
-                           json_t *contract_terms,
-                           uint64_t *order_serial);
+  (*insert_contract_terms)(
+    void *cls,
+    const char *instance_id,
+    const char *order_id,
+    json_t *contract_terms,
+    uint64_t *order_serial);
 
 
   /**
@@ -2440,6 +2482,7 @@ struct TALER_MERCHANTDB_Plugin
    * @param[out] total_picked_up how much of the tip was so far picked up (with fees)
    * @param[out] expiration set to when the tip expires
    * @param[out] exchange_url set to the exchange URL where the reserve is
+   * @param[out] next_url set to the URL where the wallet should navigate to after getting the tip
    * @param[out] reserve_priv set to private key of reserve to be debited
    * @return transaction status
    */
@@ -2451,6 +2494,7 @@ struct TALER_MERCHANTDB_Plugin
                 struct TALER_Amount *total_picked_up,
                 struct GNUNET_TIME_Timestamp *expiration,
                 char **exchange_url,
+                char **next_url,
                 struct TALER_ReservePrivateKeyP *reserve_priv);
 
 
@@ -2752,22 +2796,6 @@ struct TALER_MERCHANTDB_Plugin
                             const char *http_method,
                             const char *header,
                             const char *body);
-
-  /**
-   * Lookup details about a particular pending webhook.
-   *
-   * @param cls closure
-   * @param instance_id instance to lookup webhook for
-   * @param webhook_serial webhook to lookup
-   * @param[out] pwb set to the pending webhook details on success, can be NULL
-   *             (in that case we only want to check if the webhook exists)
-   * @return database result code
-   */
-  enum GNUNET_DB_QueryStatus
-  (*lookup_pending_webhook)(void *cls,
-                            const char *instance_id,
-                            uint64_t *webhook_serial,
-                            struct TALER_MERCHANTDB_PendingWebhookDetails *pwb);
   /**
    * Lookup the webhook that need to be send in priority. These webhooks are not successfully
    * send.
@@ -2827,9 +2855,9 @@ struct TALER_MERCHANTDB_Plugin
    */
   enum GNUNET_DB_QueryStatus
   (*update_pending_webhook)(void *cls,
-                            uint64_t webhook_serial,
+                            uint64_t webhook_pending_serial,
                             struct GNUNET_TIME_Absolute next_attempt);
-                            // maybe add: http status of failure?
+  // maybe add: http status of failure?
 
 
   /**
@@ -2842,7 +2870,7 @@ struct TALER_MERCHANTDB_Plugin
    */
   enum GNUNET_DB_QueryStatus
   (*delete_pending_webhook)(void *cls,
-                            uint64_t webhook_serial);
+                            uint64_t webhook_pending_serial);
 
 };
 
