@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2020 Taler Systems SA
+  Copyright (C) 2020-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -18,10 +18,12 @@
  * @file merchant_api_common.c
  * @brief Implementation of common logic for libtalermerchant
  * @author Christian Grothoff
+ * @author Priscilla Huang
  */
 #include "platform.h"
 #include <curl/curl.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_common.h"
 #include <gnunet/gnunet_uri_lib.h>
 #include <taler/taler_json_lib.h>
 
@@ -105,31 +107,7 @@ TALER_MERCHANT_parse_error_details_ (const json_t *response,
 }
 
 
-char *
-TALER_MERCHANT_baseurl_add_instance (const char *base_url,
-                                     const char *instance_id)
-{
-  char *ret;
-  bool end_sl;
-
-  if ('\0' == *base_url)
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
-  end_sl = '/' == base_url[strlen (base_url) - 1];
-
-  GNUNET_asprintf (&ret,
-                   (end_sl)
-                   ? "%sinstances/%s/"
-                   : "%s/instances/%s/",
-                   base_url,
-                   instance_id);
-  return ret;
-}
-
-
-int
+enum GNUNET_GenericReturnValue
 TALER_MERCHANT_parse_pay_uri (const char *pay_uri,
                               struct TALER_MERCHANT_PayUriData *parse_data)
 {
@@ -371,3 +349,138 @@ TALER_MERCHANT_parse_refund_uri_free (
   GNUNET_free (parse_data->order_id);
   GNUNET_free (parse_data->ssid);
 }
+
+
+void
+TALER_MERCHANT_handle_order_creation_response_ (
+  TALER_MERCHANT_PostOrdersCallback cb,
+  void *cb_cls,
+  long response_code,
+  const json_t *json)
+{
+  struct TALER_MERCHANT_PostOrdersReply por = {
+    .hr.http_status = (unsigned int) response_code,
+    .hr.reply = json
+  };
+  struct TALER_ClaimTokenP token;
+
+  switch (response_code)
+  {
+  case 0:
+    por.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    break;
+  case MHD_HTTP_OK:
+    {
+      bool no_token;
+      struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_string ("order_id",
+                                 &por.details.ok.order_id),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_fixed_auto ("token",
+                                       &token),
+          &no_token),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (json,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        por.hr.http_status = 0;
+        por.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+      }
+      else
+      {
+        if (! no_token)
+          por.details.ok.token = &token;
+      }
+    }
+    break;
+  case MHD_HTTP_BAD_REQUEST:
+    json_dumpf (json,
+                stderr,
+                JSON_INDENT (2));
+    por.hr.ec = TALER_JSON_get_error_code (json);
+    por.hr.hint = TALER_JSON_get_error_hint (json);
+    /* This should never happen, either us or
+       the merchant is buggy (or API version conflict);
+       just pass JSON reply to the application */
+    break;
+  case MHD_HTTP_UNAUTHORIZED:
+    por.hr.ec = TALER_JSON_get_error_code (json);
+    por.hr.hint = TALER_JSON_get_error_hint (json);
+    /* Nothing really to verify, merchant says we need to authenticate. */
+    break;
+  case MHD_HTTP_FORBIDDEN:
+    /* Nothing really to verify, merchant says one
+       of the signatures is invalid; as we checked them,
+       this should never happen, we should pass the JSON
+       reply to the application */
+    por.hr.ec = TALER_JSON_get_error_code (json);
+    por.hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  case MHD_HTTP_NOT_FOUND:
+    /* Nothing really to verify, this should never
+       happen, we should pass the JSON reply to the application */
+    por.hr.ec = TALER_JSON_get_error_code (json);
+    por.hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  case MHD_HTTP_CONFLICT:
+    por.hr.ec = TALER_JSON_get_error_code (json);
+    por.hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  case MHD_HTTP_GONE:
+    /* The quantity of some product requested was not available. */
+    {
+
+      struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_string (
+          "product_id",
+          &por.details.gone.product_id),
+        GNUNET_JSON_spec_uint64 (
+          "requested_quantity",
+          &por.details.gone.requested_quantity),
+        GNUNET_JSON_spec_uint64 (
+          "available_quantity",
+          &por.details.gone.available_quantity),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_timestamp (
+            "restock_expected",
+            &por.details.gone.restock_expected),
+          NULL),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (json,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        por.hr.http_status = 0;
+        por.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+      }
+      break;
+    }
+  case MHD_HTTP_INTERNAL_SERVER_ERROR:
+    /* Server had an internal issue; we should retry,
+       but this API leaves this to the application */
+    por.hr.ec = TALER_JSON_get_error_code (json);
+    por.hr.hint = TALER_JSON_get_error_hint (json);
+    break;
+  default:
+    /* unexpected response code */
+    por.hr.ec = TALER_JSON_get_error_code (json);
+    por.hr.hint = TALER_JSON_get_error_hint (json);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Unexpected response code %u/%d\n",
+                (unsigned int) response_code,
+                (int) por.hr.ec);
+    GNUNET_break_op (0);
+    break;
+  } // end of switch
+  cb (cb_cls,
+      &por);
+}
