@@ -26,6 +26,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 
@@ -113,15 +114,15 @@ handle_get_instance_finished (void *cls,
         GNUNET_JSON_spec_json ("jurisdiction",
                                &jurisdiction),
         TALER_JSON_spec_amount_any ("default_max_wire_fee",
-                                &default_max_wire_fee),
+                                    &default_max_wire_fee),
         GNUNET_JSON_spec_uint32 ("default_wire_fee_amortization",
                                  &default_wire_fee_amortization),
         TALER_JSON_spec_amount_any ("default_max_deposit_fee",
-                                &default_max_deposit_fee),
-        TALER_JSON_spec_relative_time ("default_wire_transfer_delay",
-                                       &default_wire_transfer_delay),
-        TALER_JSON_spec_relative_time ("default_pay_delay",
-                                       &default_pay_delay),
+                                    &default_max_deposit_fee),
+        GNUNET_JSON_spec_relative_time ("default_wire_transfer_delay",
+                                        &default_wire_transfer_delay),
+        GNUNET_JSON_spec_relative_time ("default_pay_delay",
+                                        &default_pay_delay),
         GNUNET_JSON_spec_end ()
       };
 
@@ -259,11 +260,7 @@ TALER_MERCHANT_instance_get (struct GNUNET_CURL_Context *ctx,
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Requesting URL '%s'\n",
               igh->url);
-  eh = curl_easy_init ();
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   igh->url));
+  eh = TALER_MERCHANT_curl_easy_get_ (igh->url);
   igh->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   &handle_get_instance_finished,
