@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2022 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -162,6 +162,18 @@ const char *order_1_forgets_3[] = {
   NULL
 };
 
+/**
+ * Execute the taler-merchant-webhook command with
+ * our configuration file.
+ *
+ * @param label label to use for the command.
+ */
+static struct TALER_TESTING_Command
+cmd_webhook (const char *label)
+{
+  return TALER_TESTING_cmd_webhook (label, config_file);
+}
+
 
 /**
  * Execute the taler-exchange-wirewatch command with
@@ -306,7 +318,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_orders2 ("create-proposal-1",
                                              merchant_url,
                                              MHD_HTTP_OK,
-                                             "1", /* order ID */
+                                             "1", 
                                              GNUNET_TIME_UNIT_ZERO_TS,
                                              GNUNET_TIME_UNIT_FOREVER_TS,
                                              true,
@@ -315,10 +327,17 @@ run (void *cls,
                                              "",
                                              "",
                                              NULL),
+    TALER_TESTING_cmd_merchant_post_webhooks ("post-webhooks-pay-w1",
+                                              merchant_url,
+                                              "webhook-pay-1",
+                                              "pay",
+                                              MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_testserver ("launch-http-server-for-webhooks",
+                                  12345),
     TALER_TESTING_cmd_merchant_post_orders2 ("create-proposal-1-idem",
                                              merchant_url,
                                              MHD_HTTP_OK,
-                                             "1", /* order ID */
+                                             "1", 
                                              GNUNET_TIME_UNIT_ZERO_TS,
                                              GNUNET_TIME_UNIT_FOREVER_TS,
                                              true,
@@ -330,7 +349,7 @@ run (void *cls,
     TALER_TESTING_cmd_merchant_post_orders2 ("create-proposal-1x",
                                              merchant_url,
                                              MHD_HTTP_OK,
-                                             "1x", /* order ID */
+                                             "1x",
                                              GNUNET_TIME_UNIT_ZERO_TS,
                                              GNUNET_TIME_UNIT_FOREVER_TS,
                                              true,
@@ -414,6 +433,13 @@ run (void *cls,
                                                   MHD_HTTP_OK,
                                                   NULL,
                                                   "poll-order-wallet-start-1"),
+      /* Check for webhook */
+         cmd_webhook ("pending-webhooks-pay-w1"),
+    /* Check webhook did anything: have a command that inspects traits of the testserver
+       and check if the traits have the right values set! */
+    TALER_TESTING_cmd_checkserver ("check-http-server-for-webhooks",
+                                   "launch-http-server-for-webhooks",
+                                   0),
     /* Here we expect to run into a timeout, as we do not pay this one */
     TALER_TESTING_cmd_wallet_poll_order_conclude2 ("poll-order-1x-conclude",
                                                    MHD_HTTP_PAYMENT_REQUIRED,
@@ -472,10 +498,10 @@ run (void *cls,
                                                merchant_url,
                                                PAYTO_I1,
                                                "EUR:4.98",
-                                               NULL /* random WTID */,
-                                               /* non-routable IP address,
-                                                  so we are sure to not get
-                                                  any reply */
+                                               NULL ,
+                                               /*non-routable IP address
+                                               so we are sure to not get
+                                               any reply*/
                                                "http://192.0.2.1/404/",
                                                MHD_HTTP_GATEWAY_TIMEOUT),
     TALER_TESTING_cmd_merchant_get_transfers ("get-transfers-1",
@@ -604,12 +630,12 @@ run (void *cls,
                                              "order-p3",
                                              GNUNET_TIME_UNIT_ZERO_TS,
                                              GNUNET_TIME_UNIT_FOREVER_TS,
-                                             true, /* claim token */
+                                             true, 
                                              "EUR:5.0",
                                              "unsupported-wire-method",
                                              "product-3/2",
                                              "", /* locks */
-                                             NULL /* duplicate_of */),
+                                             NULL),
     TALER_TESTING_cmd_merchant_post_orders2 ("create-proposal-p3-pd-nx",
                                              merchant_url,
                                              MHD_HTTP_NOT_FOUND,
@@ -658,10 +684,13 @@ run (void *cls,
                                             GNUNET_TIME_UNIT_ZERO_TS,
                                             GNUNET_TIME_UNIT_FOREVER_TS,
                                             "EUR:5.0"),
+    TALER_TESTING_cmd_merchant_delete_webhook ("post-webhooks-pay-w1",
+                                               merchant_url,
+                                               "webhook-pay-1",
+                                               MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_check_bank_empty ("check_bank_empty-2"),
     TALER_TESTING_cmd_end ()
   };
-
   struct TALER_TESTING_Command double_spending[] = {
     TALER_TESTING_cmd_merchant_post_orders ("create-proposal-2",
                                             merchant_url,
@@ -1335,9 +1364,9 @@ run (void *cls,
       "another template",
       "data:image/jpeg;base64,RAWDATA",
       GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_uint64 ("minimum_age", 0),
-      GNUNET_JSON_pack_time_rel ("pay_duration",
-                                 GNUNET_TIME_UNIT_MINUTES)),
+        GNUNET_JSON_pack_uint64 ("minimum_age", 0),
+        GNUNET_JSON_pack_time_rel ("pay_duration",
+                                   GNUNET_TIME_UNIT_MINUTES)),
       MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_get_template ("get-template-t2",
                                              merchant_url,
@@ -1356,9 +1385,9 @@ run (void *cls,
       "updated template",
       "data:image/jpeg;base64,RAWDATA",
       GNUNET_JSON_PACK (
-      GNUNET_JSON_pack_uint64 ("minimum_age", 0),
-      GNUNET_JSON_pack_time_rel ("pay_duration",
-                                 GNUNET_TIME_UNIT_MINUTES)),
+        GNUNET_JSON_pack_uint64 ("minimum_age", 0),
+        GNUNET_JSON_pack_time_rel ("pay_duration",
+                                   GNUNET_TIME_UNIT_MINUTES)),
       MHD_HTTP_NOT_FOUND),
     TALER_TESTING_cmd_merchant_post_templates2 (
       "post-templates-t3-amount",
@@ -1367,11 +1396,11 @@ run (void *cls,
       "a different template with an amount",
       NULL,
       GNUNET_JSON_PACK (
-                        GNUNET_JSON_pack_uint64 ("minimum_age", 0),
-                        GNUNET_JSON_pack_time_rel ("pay_duration",
-                                                   GNUNET_TIME_UNIT_MINUTES),
-                        GNUNET_JSON_pack_string ("amount",
-                                                 "EUR:4")),
+        GNUNET_JSON_pack_uint64 ("minimum_age", 0),
+        GNUNET_JSON_pack_time_rel ("pay_duration",
+                                   GNUNET_TIME_UNIT_MINUTES),
+        GNUNET_JSON_pack_string ("amount",
+                                 "EUR:4")),
       MHD_HTTP_NO_CONTENT),
     TALER_TESTING_cmd_merchant_post_using_templates (
       "using-templates-t1",
@@ -1465,7 +1494,7 @@ run (void *cls,
                                               merchant_url,
                                               "webhook-2",
                                               "Refund2",
-                                              "https://example.com",
+                                              "http://localhost:38188/",
                                               "POST",
                                               "Authorization:WHWOXZXPLL",
                                               "Amount",
@@ -1839,22 +1868,22 @@ run (void *cls,
                                                MHD_HTTP_CONFLICT),
     TALER_TESTING_cmd_batch ("pay",
                              pay),
-    TALER_TESTING_cmd_batch ("double-spending",
-                             double_spending),
+     TALER_TESTING_cmd_batch ("double-spending",
+                          double_spending),
     TALER_TESTING_cmd_batch ("pay-again",
-                             pay_again),
-    TALER_TESTING_cmd_batch ("pay-abort",
-                             pay_abort),
+                           pay_again),
+     TALER_TESTING_cmd_batch ("pay-abort",
+                           pay_abort),
     TALER_TESTING_cmd_batch ("refund",
-                             refund),
+                            refund),
     TALER_TESTING_cmd_batch ("tip",
-                             tip),
-    TALER_TESTING_cmd_batch ("auth",
-                             auth),
-    TALER_TESTING_cmd_batch ("templates",
-                             templates),
+                           tip),
+     TALER_TESTING_cmd_batch ("auth",
+                           auth),
+     TALER_TESTING_cmd_batch ("templates",
+                              templates),
     TALER_TESTING_cmd_batch ("webhooks",
-                             webhooks),
+                           webhooks),
     /**
      * End the suite.
      */
