@@ -15,7 +15,7 @@ function wire_funds() {
 
 echo -n "First prepare wallet with coins ..."
 rm $WALLET_DB
-taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api 'withdrawTestBalance' \
+taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api --expect-success 'withdrawTestBalance' \
   "$(jq -n '
     {
         amount: "TESTKUDOS:99",
@@ -96,7 +96,7 @@ then
     exit 1
 fi
 
-echo OK
+echo "OK"
 
 echo -n "Checking created order without TOKEN..."
 
@@ -110,7 +110,7 @@ then
     echo 'should have a payuri. got:' $PAY_URI `cat $LAST_RESPONSE`
     exit 1
 fi
-echo OK
+echo "OK"
 
 #
 # CREATE AN ORDER WITHOUT TOKEN WITH FULLFILMENT URL
@@ -133,6 +133,7 @@ TOKEN=`jq -r .token < $LAST_RESPONSE`
 if [ "$TOKEN" != "null" ]
 then
     echo 'token should be null, got:' $TOKEN
+    exit 1
 fi
 
 STATUS=$(curl http://localhost:9966/orders/$ORDER_ID \
@@ -152,7 +153,7 @@ then
     echo 'should have a payuri. got:' $PAY_URI `cat $LAST_RESPONSE`
     exit 1
 fi
-echo OK
+echo "OK"
 
 #
 # CREATE ORDER WITH NON-INVENTORY AND CHECK
