@@ -17,6 +17,8 @@
 # <http://www.gnu.org/licenses/>
 #
 
+set -eu
+
 . initialize_taler_system.sh
 
 echo -n "Configuring merchant instance ..."
@@ -26,31 +28,28 @@ FORTYTHREE=`get_payto_uri fortythree x`
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H 'Authorization: Bearer secret-token:super_secret' \
     http://localhost:9966/management/instances \
-    -d '{"auth":{"method":"external"},"payto_uris":["payto://x-taler-bank/'$BANK_URL'/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_us" : 50000000},"default_pay_delay":{"d_us": 60000000}}' \
+    -d '{"auth":{"method":"external"},"payto_uris":["payto://x-taler-bank/localhost:1'$NEXUS_PORT'/43"],"id":"default","name":"default","address":{},"jurisdiction":{},"default_max_wire_fee":"TESTKUDOS:1", "default_max_deposit_fee":"TESTKUDOS:1","default_wire_fee_amortization":1,"default_wire_transfer_delay":{"d_us" : 50000000},"default_pay_delay":{"d_us": 60000000}}' \
     -w "%{http_code}" -s -o /dev/null)
 
 if [ "$STATUS" != "204" ]
 then
-    echo 'should respond ok, instance created. got:' $STATUS
+    echo 'Expected 204, instance created. Got instead: ' $STATUS
     exit 1
 fi
+echo "OK"
 
-echo OK
-
-echo -n "creating reserve ..."
-
+echo -n "Creating reserve ..."
 STATUS=$(curl 'http://localhost:9966/instances/default/private/reserves' \
     -d '{"initial_balance":"TESTKUDOS:2","exchange_url":"'$EXCHANGE_URL'","wire_method":"iban"}' \
     -w "%{http_code}" -s -o $LAST_RESPONSE)
 
-
 if [ "$STATUS" != "200" ]
 then
-    echo 'should respond ok, reserve created. got:' $STATUS
+    echo 'Expected 200, reserve created. Got instead: ' $STATUS
     exit 1
 fi
 
-echo OK
+echo "OK"
 
 RESERVE_PUB=`jq -r .reserve_pub < $LAST_RESPONSE`
 
@@ -61,35 +60,36 @@ FUNDED=`jq -r '.merchant_initial_amount == .exchange_initial_amount' < $LAST_RES
 
 if [ "$FUNDED" != "false" ]
 then
-    echo 'should not be funded if we just created. got:' $STATUS 'is founded: ' $FUNDED
+    echo 'Should not yet be funded if we just created. Got:' $STATUS 'is founded: ' $FUNDED
     cat $LAST_RESPONSE
     exit 1
 fi
 
 
-echo -n Wire transferring...
+echo -n "Wire transferring... "
 # Exchange wants TESTKUDOS:2 from account 43, under RESERVE_PUB.
 
 EXCHANGE_PAYTO=`get_payto_uri exchange x`
 export LIBEUFIN_SANDBOX_USERNAME=fortythree
 export LIBEUFIN_SANDBOX_PASSWORD=x
-export LIBEUFIN_SANDBOX_URL=http://localhost:18082/demobanks/default
+export LIBEUFIN_SANDBOX_URL="http://localhost:18082/"
 libeufin-cli sandbox demobank new-transaction \
   --bank-account fortythree \
   --payto-with-subject "$EXCHANGE_PAYTO&message=$RESERVE_PUB" \
-  --amount 2 # Currency taken from the demobank config.
+  --amount TESTKUDOS:2
 unset LIBEUFIN_SANDBOX_USERNAME
 unset LIBEUFIN_SANDBOX_PASSWORD
 unset LIBEUFIN_SANDBOX_URL
-echo -n "Give background tasks time to detect the payment..."
-sleep 4
-echo " OK"
+echo "OK"
+echo -n "Give Nexus time to detect the payment... "
+sleep 10 # FIXME-MS: replace with call to Nexus to right now poll the sandbox ...
+echo "OK"
 
 # Stop existing background service, we need to run it here, now, and only once
 kill -TERM $WIREWATCH_PID
 wait $WIREWATCH_PID
 
-taler-exchange-wirewatch -c $CONF -t -L INFO
+taler-exchange-wirewatch -c $CONF -t -L INFO &> taler-exchange-wirewatch.log
 
 STATUS=$(curl 'http://localhost:9966/instances/default/private/reserves/'$RESERVE_PUB \
     -w "%{http_code}" -s -o $LAST_RESPONSE)
