@@ -102,7 +102,7 @@ struct PatchProductState
   /**
    * when the next restocking is expected to happen, 0 for unknown,
    */
-  struct GNUNET_TIME_Absolute next_restock;
+  struct GNUNET_TIME_Timestamp next_restock;
 
   /**
    * Expected HTTP response code.
@@ -201,7 +201,7 @@ patch_product_run (void *cls,
  * @param index index number of the object to extract.
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 patch_product_traits (void *cls,
                       const void **ret,
                       const char *trait,
@@ -209,16 +209,18 @@ patch_product_traits (void *cls,
 {
   struct PatchProductState *pps = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_string (0, pps->description),
-    TALER_TESTING_make_trait_json (0, pps->description_i18n),
-    TALER_TESTING_make_trait_string (1, pps->unit),
-    TALER_TESTING_make_trait_amount_obj (0, &pps->price),
-    TALER_TESTING_make_trait_string (3, pps->image),
-    TALER_TESTING_make_trait_json (2, pps->taxes),
-    TALER_TESTING_make_trait_int64 (0, &pps->total_stock),
-    TALER_TESTING_make_trait_json (3, pps->address),
-    TALER_TESTING_make_trait_absolute_time (0, &pps->next_restock),
-    TALER_TESTING_make_trait_string (2, pps->product_id),
+    TALER_TESTING_make_trait_product_description (&pps->description),
+    TALER_TESTING_make_trait_i18n_description (pps->description_i18n),
+    TALER_TESTING_make_trait_product_unit (&pps->unit),
+    TALER_TESTING_make_trait_amount (&pps->price),
+    TALER_TESTING_make_trait_product_image (
+      (const char **) &pps->image),
+    TALER_TESTING_make_trait_taxes (pps->taxes),
+    TALER_TESTING_make_trait_product_stock (&pps->total_stock),
+    TALER_TESTING_make_trait_address (pps->address),
+    TALER_TESTING_make_trait_timestamp (0,
+                                        &pps->next_restock),
+    TALER_TESTING_make_trait_product_id (&pps->product_id),
     TALER_TESTING_trait_end (),
   };
 
@@ -270,7 +272,7 @@ TALER_TESTING_cmd_merchant_patch_product (
   int64_t total_stock,
   uint64_t total_lost,
   json_t *address,
-  struct GNUNET_TIME_Absolute next_restock,
+  struct GNUNET_TIME_Timestamp next_restock,
   unsigned int http_status)
 {
   struct PatchProductState *pis;
