@@ -133,7 +133,7 @@ post_reserves_cb (void *cls,
  * @param index index number of the object to extract.
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 post_reserves_traits (void *cls,
                       const void **ret,
                       const char *trait,
@@ -141,8 +141,8 @@ post_reserves_traits (void *cls,
 {
   struct PostReservesState *prs = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_reserve_pub (0, &prs->reserve_pub),
-    TALER_TESTING_make_trait_amount_obj (0, &prs->initial_balance),
+    TALER_TESTING_make_trait_reserve_pub (&prs->reserve_pub),
+    TALER_TESTING_make_trait_amount (&prs->initial_balance),
     TALER_TESTING_trait_end (),
   };
 
