@@ -76,12 +76,12 @@ struct TipAuthorizeState
   /**
    * The tip id; set when the CMD succeeds.
    */
-  struct GNUNET_HashCode tip_id;
+  struct TALER_TipIdentifierP tip_id;
 
   /**
    * Expiration date for this tip.
    */
-  struct GNUNET_TIME_Absolute tip_expiration;
+  struct GNUNET_TIME_Timestamp tip_expiration;
 
   /**
    * Handle to the on-going /tip-authorize request.
@@ -134,9 +134,9 @@ do_retry (void *cls);
 static void
 tip_authorize_cb (void *cls,
                   const struct TALER_MERCHANT_HttpResponse *hr,
-                  struct GNUNET_HashCode *tip_id,
+                  struct TALER_TipIdentifierP *tip_id,
                   const char *taler_tip_uri,
-                  struct GNUNET_TIME_Absolute expiration)
+                  struct GNUNET_TIME_Timestamp expiration)
 {
   struct TipAuthorizeState *tas = cls;
 
@@ -198,28 +198,26 @@ tip_authorize_cb (void *cls,
  * @param index index number of the object to extract.
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 tip_authorize_traits (void *cls,
                       const void **ret,
                       const char *trait,
                       unsigned int index)
 {
   struct TipAuthorizeState *tas = cls;
+  struct TALER_TESTING_Trait traits[] = {
+    TALER_TESTING_make_trait_tip_id (&tas->tip_id),
+    TALER_TESTING_make_trait_amount (&tas->amount),
+    TALER_TESTING_make_trait_reason (&tas->justification),
+    TALER_TESTING_make_trait_timestamp (0,
+                                        &tas->tip_expiration),
+    TALER_TESTING_trait_end (),
+  };
 
-  {
-    struct TALER_TESTING_Trait traits[] = {
-      TALER_TESTING_make_trait_tip_id (0, &tas->tip_id),
-      TALER_TESTING_make_trait_amount_obj (0, &tas->amount),
-      TALER_TESTING_make_trait_string (0, tas->justification),
-      TALER_TESTING_make_trait_absolute_time (0, &tas->tip_expiration),
-      TALER_TESTING_trait_end (),
-    };
-
-    return TALER_TESTING_get_trait (traits,
-                                    ret,
-                                    trait,
-                                    index);
-  }
+  return TALER_TESTING_get_trait (traits,
+                                  ret,
+                                  trait,
+                                  index);
 }
 
 
@@ -270,7 +268,6 @@ do_retry (void *cls)
       tas->reserve_reference);
     GNUNET_assert (GNUNET_OK ==
                    TALER_TESTING_get_trait_reserve_pub (reserve_cmd,
-                                                        0,
                                                         &reserve_pub));
     tas->tao = TALER_MERCHANT_tip_authorize2 (tas->is->ctx,
                                               tas->merchant_url,
@@ -302,8 +299,7 @@ tip_authorize_fake_run (void *cls,
   /* Make up a tip id.  */
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                               &tas->tip_id,
-                              sizeof (struct GNUNET_HashCode));
-
+                              sizeof (struct TALER_TipIdentifierP));
   TALER_TESTING_interpreter_next (is);
 }
 
