@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -22,7 +22,6 @@
  * @brief command to test the tipping.
  * @author Marcello Stanisci
  */
-
 #include "platform.h"
 #include <taler/taler_exchange_service.h>
 #include <taler/taler_testing_lib.h>
@@ -89,11 +88,6 @@ struct TipPickupState
    */
   struct TALER_Amount total_amount;
 
-  /**
-   * How many coins are involved in the tipping operation.
-   */
-  uint64_t num_coins;
-
   /**
    * The array of denomination keys, in the same order of @a
    * amounts.
@@ -104,13 +98,18 @@ struct TipPickupState
    * The array of planchet secrets, in the same order of @a
    * amounts.
    */
-  struct TALER_PlanchetSecretsP *psa;
+  struct TALER_PlanchetMasterSecretP *psa;
 
   /**
    * Set (by the interpreter) to an array of @a num_coins
-   * signatures created from the (successful) tip operation.
+   * details on coins created from the (successful) tip operation.
    */
-  struct TALER_DenominationSignature *sigs;
+  struct TALER_EXCHANGE_PrivateCoinDetails *pcds;
+
+  /**
+   * How many coins are involved in the tipping operation.
+   */
+  uint32_t num_coins;
 
   /**
    * Expected Taler error code (NOTE: this is NOT the HTTP
@@ -126,47 +125,48 @@ struct TipPickupState
  * (and if the status was 200 OK) proceede with the withdrawal.
  *
  * @param cls closure
- * @param hr HTTP response
- * @param num_sigs length of the @a sigs array,
- *        0 on error
- * @param sigs array of signatures over the coins, NULL on error
+ * @param pd details about the result of the operation
  */
 static void
 pickup_cb (void *cls,
-           const struct TALER_MERCHANT_HttpResponse *hr,
-           unsigned int num_sigs,
-           const struct TALER_DenominationSignature *sigs)
+           const struct TALER_MERCHANT_PickupDetails *pd)
 {
   struct TipPickupState *tps = cls;
 
   tps->tpo = NULL;
-  if (hr->http_status != tps->http_status)
+  if (pd->hr.http_status != tps->http_status)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u (%d) to command %s\n",
-                hr->http_status,
-                (int) hr->ec,
+                pd->hr.http_status,
+                (int) pd->hr.ec,
                 TALER_TESTING_interpreter_get_current_label (tps->is));
     TALER_TESTING_FAIL (tps->is);
   }
 
-  if (hr->ec != tps->expected_ec)
+  if (pd->hr.ec != tps->expected_ec)
     TALER_TESTING_FAIL (tps->is);
 
   /* Safe to go ahead: http status was expected.  */
-  if ( (MHD_HTTP_OK != hr->http_status) ||
-       (TALER_EC_NONE != hr->ec) )
+  if ( (MHD_HTTP_OK != pd->hr.http_status) ||
+       (TALER_EC_NONE != pd->hr.ec) )
   {
     TALER_TESTING_interpreter_next (tps->is);
     return;
   }
-  if (num_sigs != tps->num_coins)
+  if (pd->details.success.num_sigs != tps->num_coins)
     TALER_TESTING_FAIL (tps->is);
-  tps->sigs = GNUNET_new_array (tps->num_coins,
-                                struct TALER_DenominationSignature);
-  for (unsigned int i = 0; i<num_sigs; i++)
-    tps->sigs[i].rsa_signature
-      = GNUNET_CRYPTO_rsa_signature_dup (sigs[i].rsa_signature);
+  tps->pcds = GNUNET_new_array (tps->num_coins,
+                                struct TALER_EXCHANGE_PrivateCoinDetails);
+  for (unsigned int i = 0; i<tps->num_coins; i++)
+  {
+    struct TALER_EXCHANGE_PrivateCoinDetails *pcd =
+      &pd->details.success.pcds[i];
+
+    tps->pcds[i] = *pcd;
+    TALER_denom_sig_deep_copy (&tps->pcds[i].sig,
+                               &pcd->sig);
+  }
   TALER_TESTING_interpreter_next (tps->is);
 }
 
@@ -187,7 +187,7 @@ tip_pickup_run (void *cls,
   unsigned int num_planchets;
   const struct TALER_TESTING_Command *replay_cmd;
   const struct TALER_TESTING_Command *authorize_cmd;
-  const struct GNUNET_HashCode *tip_id;
+  const struct TALER_TipIdentifierP *tip_id;
 
   tps->is = is;
   tps->exchange_url = TALER_EXCHANGE_get_base_url (is->exchange);
@@ -203,7 +203,7 @@ tip_pickup_run (void *cls,
   }
   else
   {
-    const unsigned int *np;
+    const uint32_t *np;
 
     if (NULL ==  /* looking for "parent" tip-pickup command */
         (replay_cmd
@@ -212,9 +212,8 @@ tip_pickup_run (void *cls,
       TALER_TESTING_FAIL (is);
 
     if (GNUNET_OK !=
-        TALER_TESTING_get_trait_uint (replay_cmd,
-                                      0,
-                                      &np))
+        TALER_TESTING_get_trait_num_planchets (replay_cmd,
+                                               &np))
       TALER_TESTING_FAIL (is);
     num_planchets = *np;
   }
@@ -230,7 +229,7 @@ tip_pickup_run (void *cls,
     struct TALER_MERCHANT_PlanchetData planchets[num_planchets];
 
     tps->psa = GNUNET_new_array (num_planchets,
-                                 struct TALER_PlanchetSecretsP);
+                                 struct TALER_PlanchetMasterSecretP);
     tps->dks = GNUNET_new_array (num_planchets,
                                  const struct TALER_EXCHANGE_DenomPublicKey *);
     tps->amounts_obj = GNUNET_new_array (num_planchets,
@@ -252,14 +251,15 @@ tip_pickup_run (void *cls,
                                          &tps->total_amount,
                                          &tps->amounts_obj[i]));
         tps->dks[i] = TALER_TESTING_find_pk (is->keys,
-                                             &tps->amounts_obj[i]);
+                                             &tps->amounts_obj[i],
+                                             false);
         if (NULL == tps->dks[i])
           TALER_TESTING_FAIL (is);
-        TALER_planchet_setup_random (&tps->psa[i]);
+        TALER_planchet_master_setup_random (&tps->psa[i]);
       }
       else
       {
-        struct TALER_PlanchetSecretsP *ps;
+        const struct TALER_PlanchetMasterSecretP *ps;
 
         if (GNUNET_OK !=
             TALER_TESTING_get_trait_denom_pub (replay_cmd,
@@ -278,11 +278,10 @@ tip_pickup_run (void *cls,
     }
     if (GNUNET_OK !=
         TALER_TESTING_get_trait_tip_id (authorize_cmd,
-                                        0,
                                         &tip_id))
       TALER_TESTING_FAIL (is);
-
     tps->tpo = TALER_MERCHANT_tip_pickup (is->ctx,
+                                          is->exchange,
                                           tps->merchant_url,
                                           tip_id,
                                           num_planchets,
@@ -310,12 +309,11 @@ tip_pickup_cleanup (void *cls,
   GNUNET_free (tps->amounts_obj);
   GNUNET_free (tps->dks);
   GNUNET_free (tps->psa);
-  if (NULL != tps->sigs)
+  if (NULL != tps->pcds)
   {
     for (unsigned int i = 0; i<tps->num_coins; i++)
-      if (NULL != tps->sigs[i].rsa_signature)
-        GNUNET_CRYPTO_rsa_signature_free (tps->sigs[i].rsa_signature);
-    GNUNET_free (tps->sigs);
+      TALER_denom_sig_free (&tps->pcds[i].sig);
+    GNUNET_free (tps->pcds);
   }
   if (NULL != tps->tpo)
   {
@@ -326,43 +324,39 @@ tip_pickup_cleanup (void *cls,
 }
 
 
-static int
+static enum GNUNET_GenericReturnValue
 tip_pickup_traits (void *cls,
                    const void **ret,
                    const char *trait,
                    unsigned int index)
 {
   struct TipPickupState *tps = cls;
-  #define NUM_TRAITS (tps->num_coins * 5) + 4
-  struct TALER_TESTING_Trait traits[NUM_TRAITS];
 
-  for (unsigned int i = 0; i<tps->num_coins; i++)
+  if (index >= tps->num_coins)
+    return GNUNET_SYSERR;
   {
-    traits[i] = TALER_TESTING_make_trait_planchet_secrets (i,
-                                                           &tps->psa[i]);
-    traits[i + tps->num_coins] =
-      TALER_TESTING_make_trait_coin_priv (i, &tps->psa[i].coin_priv);
-    traits[i + (tps->num_coins * 2)] =
-      TALER_TESTING_make_trait_denom_pub (i, tps->dks[i]);
-    traits[i + (tps->num_coins * 3)] =
-      TALER_TESTING_make_trait_denom_sig (i, &tps->sigs[i]);
-    traits[i + (tps->num_coins * 4)] =
-      TALER_TESTING_make_trait_amount_obj (i, &tps->amounts_obj[i]);
+    struct TALER_TESTING_Trait traits[] = {
+      TALER_TESTING_make_trait_planchet_secrets (index,
+                                                 &tps->psa[index]),
+      TALER_TESTING_make_trait_coin_priv (index,
+                                          &tps->pcds[index].coin_priv),
+      TALER_TESTING_make_trait_denom_pub (index,
+                                          tps->dks[index]),
+      TALER_TESTING_make_trait_denom_sig (index,
+                                          &tps->pcds[index].sig),
+      TALER_TESTING_make_trait_amounts (index,
+                                        &tps->amounts_obj[index]),
+      TALER_TESTING_make_trait_amount (&tps->total_amount),
+      TALER_TESTING_make_trait_num_planchets (&tps->num_coins),
+      TALER_TESTING_make_trait_exchange_url (&tps->exchange_url),
+      TALER_TESTING_trait_end ()
+    };
+
+    return TALER_TESTING_get_trait (traits,
+                                    ret,
+                                    trait,
+                                    index);
   }
-  traits[NUM_TRAITS - 4]
-    = TALER_TESTING_make_trait_amount_obj (tps->num_coins,
-                                           &tps->total_amount);
-  traits[NUM_TRAITS - 3]
-    = TALER_TESTING_make_trait_uint64 (0,
-                                       &tps->num_coins);
-  traits[NUM_TRAITS - 2]
-    = TALER_TESTING_make_trait_url (TALER_TESTING_UT_EXCHANGE_BASE_URL,
-                                    tps->exchange_url);
-  traits[NUM_TRAITS - 1] = TALER_TESTING_trait_end ();
-  return TALER_TESTING_get_trait (traits,
-                                  ret,
-                                  trait,
-                                  index);
 }
 
 
