@@ -79,38 +79,15 @@ struct WalletGetOrderState
  * Callback to process a GET /orders/$ID request
  *
  * @param cls closure
- * @param hr HTTP response details
- * @param paid #GNUNET_YES if the payment is settled, #GNUNET_NO if not
- *        settled, #GNUNET_SYSERR on error
- *        (note that refunded payments are returned as paid!)
- * @param refunded #GNUNET_YES if there is at least on refund on this payment,
- *        #GNUNET_NO if refunded, #GNUNET_SYSERR or error
- * @param refund_pending #GNUNET_YES if there are refunds waiting to be
- *        obtained, #GNUNET_NO if all refunds have been obtained, #GNUNET_SYSERR
- *        on error.
- * @param refund_amount amount that was refunded, NULL if there
- *        was no refund
- * @param taler_pay_uri the URI that instructs the wallets to process
- *                      the payment
- * @param already_paid_order_id equivalent order that this customer
- *                      paid already, or NULL for none
+ * @param owgr response details
  */
 static void
 wallet_get_order_cb (
   void *cls,
-  const struct TALER_MERCHANT_HttpResponse *hr,
-  enum GNUNET_GenericReturnValue paid,
-  enum GNUNET_GenericReturnValue refunded,
-  enum GNUNET_GenericReturnValue refund_pending,
-  struct TALER_Amount *refund_amount,
-  const char *taler_pay_uri,
-  const char *already_paid_order_id)
+  const struct TALER_MERCHANT_OrderWalletGetResponse *owgr)
 {
-  /* FIXME, deeper checks should be implemented here. */
   struct WalletGetOrderState *gos = cls;
-  bool paid_b = (paid == GNUNET_YES);
-  bool refunded_b = (refunded == GNUNET_YES);
-  bool refund_pending_b = (refund_pending == GNUNET_YES);
+  const struct TALER_MERCHANT_HttpResponse *hr = &owgr->hr;
 
   gos->ogh = NULL;
   if (gos->http_status != hr->http_status)
@@ -126,44 +103,36 @@ wallet_get_order_cb (
   switch (hr->http_status)
   {
   case MHD_HTTP_OK:
-    // FIXME: use gos->order_reference here to
-    // check if the data returned matches that from the POST / PATCH
-    if (gos->paid != paid_b)
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Order paid does not match\n");
-      TALER_TESTING_interpreter_fail (gos->is);
-      return;
-    }
-    if (gos->refunded != refunded_b)
+    if (gos->refunded != owgr->details.success.refunded)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Order refunded does not match\n");
       TALER_TESTING_interpreter_fail (gos->is);
       return;
     }
-    if (gos->refund_pending != refund_pending_b)
+    if (gos->refund_pending != owgr->details.success.refund_pending)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Order refund pending does not match\n");
       TALER_TESTING_interpreter_fail (gos->is);
       return;
     }
-    if (! paid_b)
+    break;
+  case MHD_HTTP_PAYMENT_REQUIRED:
     {
-      /* FIXME: Check all of the members of `pud` */
       struct TALER_MERCHANT_PayUriData pud;
       const struct TALER_TESTING_Command *order_cmd;
-      const char *order_id;
+      const char **order_id;
       const struct TALER_ClaimTokenP *claim_token;
 
       if (GNUNET_OK !=
-          TALER_MERCHANT_parse_pay_uri (taler_pay_uri,
-                                        &pud))
+          TALER_MERCHANT_parse_pay_uri (
+            owgr->details.payment_required.taler_pay_uri,
+            &pud))
       {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                     "Taler pay uri `%s' is malformed\n",
-                    taler_pay_uri);
+                    owgr->details.payment_required.taler_pay_uri);
         TALER_TESTING_interpreter_fail (gos->is);
         return;
       }
@@ -174,7 +143,6 @@ wallet_get_order_cb (
 
       if (GNUNET_OK !=
           TALER_TESTING_get_trait_order_id (order_cmd,
-                                            0,
                                             &order_id))
       {
         TALER_MERCHANT_parse_pay_uri_free (&pud);
@@ -183,7 +151,6 @@ wallet_get_order_cb (
 
       if (GNUNET_OK !=
           TALER_TESTING_get_trait_claim_token (order_cmd,
-                                               0,
                                                &claim_token))
       {
         TALER_MERCHANT_parse_pay_uri_free (&pud);
@@ -213,13 +180,13 @@ wallet_get_order_cb (
         if ((0 != strcmp (host,
                           pud.merchant_host)) ||
             (NULL != pud.merchant_prefix_path) ||
-            (0 != strcmp (order_id,
+            (0 != strcmp (*order_id,
                           pud.order_id)) ||
             (NULL != pud.ssid))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Order pay uri `%s' does not match `%s'\n",
-                      taler_pay_uri,
+                      owgr->details.payment_required.taler_pay_uri,
                       pud.order_id);
           TALER_TESTING_interpreter_fail (gos->is);
           TALER_MERCHANT_parse_pay_uri_free (&pud);
@@ -268,8 +235,8 @@ wallet_get_order_run (void *cls,
 {
   struct WalletGetOrderState *gos = cls;
   const struct TALER_TESTING_Command *order_cmd;
-  const char *order_id;
-  const struct GNUNET_HashCode *h_contract;
+  const char **order_id;
+  const struct TALER_PrivateContractHashP *h_contract;
 
   order_cmd = TALER_TESTING_interpreter_lookup_command (
     is,
@@ -277,20 +244,18 @@ wallet_get_order_run (void *cls,
 
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_order_id (order_cmd,
-                                        0,
                                         &order_id))
     TALER_TESTING_FAIL (is);
 
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_h_contract_terms (order_cmd,
-                                                0,
                                                 &h_contract))
     TALER_TESTING_FAIL (is);
 
   gos->is = is;
   gos->ogh = TALER_MERCHANT_wallet_order_get (is->ctx,
                                               gos->merchant_url,
-                                              order_id,
+                                              *order_id,
                                               h_contract,
                                               GNUNET_TIME_UNIT_ZERO,
                                               NULL,
@@ -570,54 +535,42 @@ conclude_task (void *cls)
  * Process response from a GET /orders/$ID request
  *
  * @param cls a `struct WalletPollOrderStartState *`
- * @param hr HTTP response details
- * @param paid #GNUNET_YES if the payment is settled, #GNUNET_NO if not
- *        settled, #GNUNET_SYSERR on error
- *        (note that refunded payments are returned as paid!)
- * @param refunded #GNUNET_YES if there is at least on refund on this payment,
- *        #GNUNET_NO if refunded, #GNUNET_SYSERR or error
- * @param refund_pending #GNUNET_YES if there are refunds waiting to be
- *        obtained, #GNUNET_NO if all refunds have been obtained, #GNUNET_SYSERR
- *        on error.
- * @param refund_amount amount that was refunded, NULL if there
- *        was no refund
- * @param taler_pay_uri the URI that instructs the wallets to process
- *                      the payment
- * @param already_paid_order_id equivalent order that this customer
- *                      paid already, or NULL for none
+ * @param owgr response details
  */
 static void
 wallet_poll_order_cb (
   void *cls,
-  const struct TALER_MERCHANT_HttpResponse *hr,
-  enum GNUNET_GenericReturnValue paid,
-  enum GNUNET_GenericReturnValue refunded,
-  enum GNUNET_GenericReturnValue refund_pending,
-  struct TALER_Amount *refund_amount,
-  const char *taler_pay_uri,
-  const char *already_paid_order_id)
+  const struct TALER_MERCHANT_OrderWalletGetResponse *owgr)
 {
   struct WalletPollOrderStartState *pos = cls;
+  const struct TALER_MERCHANT_HttpResponse *hr = &owgr->hr;
 
   pos->ogh = NULL;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "GET /orders/$ID finished with status %u.\n",
+              hr->http_status);
   pos->http_status = hr->http_status;
   switch (hr->http_status)
   {
   case MHD_HTTP_OK:
-    pos->paid = (GNUNET_YES == paid);
-    pos->refunded = (GNUNET_YES == refunded);
-    pos->refund_pending = (GNUNET_YES == refund_pending);
-    if (NULL != refund_amount)
-      pos->refund_available = *refund_amount;
-    if (NULL != already_paid_order_id)
-      pos->already_paid_order_id = GNUNET_strdup (already_paid_order_id);
+    pos->paid = true;
+    pos->refunded = owgr->details.success.refunded;
+    pos->refund_pending = owgr->details.success.refund_pending;
+    if (owgr->details.success.refunded)
+      pos->refund_available = owgr->details.success.refund_amount;
+    break;
+  case MHD_HTTP_PAYMENT_REQUIRED:
+    if (NULL != owgr->details.payment_required.already_paid_order_id)
+      pos->already_paid_order_id = GNUNET_strdup (
+        owgr->details.payment_required.already_paid_order_id);
     break;
   default:
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Unhandled HTTP status.\n");
     break;
   }
-  if (NULL != pos->cs)
+  if ( (NULL != pos->cs) &&
+       (NULL != pos->cs->task) )
   {
     GNUNET_SCHEDULER_cancel (pos->cs->task);
     pos->cs->task = GNUNET_SCHEDULER_add_now (&conclude_task,
@@ -640,8 +593,8 @@ wallet_poll_order_start_run (void *cls,
 {
   struct WalletPollOrderStartState *pos = cls;
   const struct TALER_TESTING_Command *order_cmd;
-  const char *order_id;
-  const struct GNUNET_HashCode *h_contract;
+  const char **order_id;
+  const struct TALER_PrivateContractHashP *h_contract;
 
   order_cmd = TALER_TESTING_interpreter_lookup_command (
     is,
@@ -649,13 +602,11 @@ wallet_poll_order_start_run (void *cls,
 
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_order_id (order_cmd,
-                                        0,
                                         &order_id))
     TALER_TESTING_FAIL (is);
 
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_h_contract_terms (order_cmd,
-                                                0,
                                                 &h_contract))
     TALER_TESTING_FAIL (is);
 
@@ -666,7 +617,7 @@ wallet_poll_order_start_run (void *cls,
   pos->is = is;
   pos->ogh = TALER_MERCHANT_wallet_order_get (is->ctx,
                                               pos->merchant_url,
-                                              order_id,
+                                              *order_id,
                                               h_contract,
                                               pos->timeout,
                                               pos->session_id,
@@ -791,6 +742,12 @@ wallet_poll_order_conclude_run (void *cls,
   GNUNET_assert (poll_cmd->run == &wallet_poll_order_start_run);
   pos = poll_cmd->cls;
   pos->cs = poc;
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Waiting on GET /orders/$ID of %s (%s)\n",
+              poc->start_reference,
+              (NULL == pos->ogh)
+              ? "finished"
+              : "active");
   if (NULL == pos->ogh)
     poc->task = GNUNET_SCHEDULER_add_now (&conclude_task,
                                           poc);
